@@ -11,6 +11,7 @@ import io
 import functools
 import logging
 import time
+import warnings
 
 try:
     import boto3
@@ -190,6 +191,33 @@ def _consolidate_params(uri, transport_params):
 
 
 def open_uri(uri, mode, transport_params):
+    deprecated = (
+        'multipart_upload_kwargs',
+        'object_kwargs',
+        'resource',
+        'resource_kwargs',
+        'session',
+        'singlepart_upload_kwargs',
+    )
+    detected = [k for k in deprecated if k in transport_params]
+    if detected:
+        doc_url = (
+            'https://github.com/RaRe-Technologies/smart_open/blob/develop/'
+            'MIGRATING_FROM_OLDER_VERSIONS.rst'
+        )
+        #
+        # We use warnings.warn /w UserWarning instead of logger.warn here because
+        #
+        # 1) Not everyone has logging enabled; and
+        # 2) check_kwargs (below) already uses logger.warn with a similar message
+        #
+        # https://github.com/RaRe-Technologies/smart_open/issues/614
+        #
+        message = (
+            'ignoring the following deprecated transport parameters: %r. '
+            'See <%s> for details' % (detected, doc_url)
+        )
+        warnings.warn(message, UserWarning)
     parsed_uri = parse_uri(uri)
     parsed_uri, transport_params = _consolidate_params(parsed_uri, transport_params)
     kwargs = smart_open.utils.check_kwargs(open, transport_params)
@@ -657,17 +685,12 @@ class Reader(io.BufferedIOBase):
         """Do nothing."""
         pass
 
-    def to_boto3(self, resource=None):
+    def to_boto3(self, resource):
         """Create an **independent** `boto3.s3.Object` instance that points to
-        the same resource as this instance.
-
-        The created instance will re-use the session and resource parameters of
-        the current instance, but it will be independent: changes to the
-        `boto3.s3.Object` may not necessarily affect the current instance.
-
+        the same S3 object as this instance.
+        Changes to the returned object will not affect the current instance.
         """
-        if resource is None:
-            resource = boto3.resource('s3')
+        assert resource, 'resource must be a boto3.resource instance'
         obj = resource.Object(self._bucket, self._key)
         if self._version_id is not None:
             return obj.Version(self._version_id)
@@ -862,17 +885,12 @@ multipart upload may fail")
         )
         self._upload_id = None
 
-    def to_boto3(self, resource=None):
+    def to_boto3(self, resource):
         """Create an **independent** `boto3.s3.Object` instance that points to
-        the same resource as this instance.
-
-        The created instance will re-use the session and resource parameters of
-        the current instance, but it will be independent: changes to the
-        `boto3.s3.Object` may not necessary affect the current instance.
-
+        the same S3 object as this instance.
+        Changes to the returned object will not affect the current instance.
         """
-        if not resource:
-            resource = boto3.resource('s3')
+        assert resource, 'resource must be a boto3.resource instance'
         return resource.Object(self._bucket, self._key)
 
     #
@@ -1222,7 +1240,7 @@ def _download_key(key_name, bucket_name=None, retries=3, **session_kwargs):
         raise ValueError('bucket_name may not be None')
 
     #
-    # https://boto3.amazonaws.com/v1/documentation/api/latest/guide/resources.html#multithreading-and-multiprocessing
+    # https://boto3.amazonaws.com/v1/documentation/api/latest/guide/resources.html#multithreading-or-multiprocessing-with-resources
     #
     session = boto3.session.Session(**session_kwargs)
     s3 = session.resource('s3')
